/*
 * MAX : HPmulti.cc, Wed Jan  2 14:44:59 CET 2002 -sg
 *
 * Device driver for HP Digitizing Oscilloscope 54XXXB 
 * 	- multi channel version !
 *
 * M. Dammasch   15.12.1998 error()-handling changed (only 0 returns)
 * 
 * This file is part of Max data acquisition software
 * (c) 1998 by M. Dammasch
 *
 * Max is free software; you can redistribute it and/or modify it
 * under the terms of the version 2 of GNU General Public License as
 * published by the Free Software Foundation.
 *
 * Max is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * (see the file COPYING) in this directory; if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

//this header has always to be included !!
#include <stdio.h>
//#include <math.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>
#include <unistd.h>
#include <ugpib.h>
#include <gpiberr.h>
#include <gpib.h>
#include "list.hh"
#include "sched.hh"
#include "error.hh"

#define HP_REFERENCE_LEFT    -1
#define HP_REFERENCE_CENTER  0
#define HP_REFERENCE_RIGHT   1

#define HP_TYPE_REALTIME     0
#define HP_TYPE_NORMAL       0
#define HP_TYPE_SINGLE       0

#define HP_TYPE_REPETITIVE   1
#define HP_TYPE_AVERAGE      1
#define HP_TYPE_TRIGGERED    1

#define HP_TYPE_ENVELOPE     2
#define HP_TYPE_RAWDATA      3

#define HP_FORMAT_COMPRESSED  0
#define HP_FORMAT_BYTE        1
#define HP_FORMAT_WORD        2
#define HP_FORMAT_ASCII       3

typedef struct {
  int    no;
  double yrange;
  double yoffset;
  int    active;
  int    enabled;
} HP_Channel;


static GPIB_Device oszi;    // gpib-device structure of the oscilloscope

static HP_Channel ch1, ch2, ch3, ch4;

static int TLAM;

static char errmsg[51];    // local error message string
static char qterrmsg[81];  // compound error message string for qt-error()-widget

static int tb_type;
static double tb_range;
static double tb_delay;
static int tb_delref;

static int aq_type;
static int aq_format;
static int aq_counts;
static int aq_points;
static int aq_complete;
static double aq_del;
static int slowmode;

///////////////////////////////////////////////////
static int HP_set_timebase ( int type, double range, double delay, int reference )
{
  char cmd[GPIB_STRMAX+1];
  
  if (type == HP_TYPE_REALTIME) sprintf(cmd,":timebase:sample realtime");
  else
  if (type == HP_TYPE_REPETITIVE) sprintf(cmd,":timebase:sample repetitive");
  else return 0;
  if ( !gpib_write(&oszi,cmd)) return 0;

  if (type == HP_TYPE_REALTIME) sprintf(cmd,":timebase:mode single");
  else
  if (type == HP_TYPE_REPETITIVE) sprintf(cmd,":timebase:mode triggered");
  else return 0;
  if ( !gpib_write(&oszi,cmd)) return 0;

  if ( !gpib_write(&oszi,":timebase:range %E",range)) return 0;

  if ( !gpib_write(&oszi,":timebase:delay %E",delay)) return 0;

  switch (reference)
  {
    case HP_REFERENCE_LEFT :      
      sprintf(cmd,":timebase:reference left");
    break;
    case HP_REFERENCE_CENTER :
      sprintf(cmd,":timebase:reference center");
    break;
    case HP_REFERENCE_RIGHT :
      sprintf(cmd,":timebase:reference right");
    break;
    default :    
      sprintf(errmsg,"\nIllegal reference (%d)\n",reference);
      return 0;
    break;
  }
  if ( !gpib_write(&oszi,cmd)) return 0;
  
  return 1;
}

static int HP_get_timebase ( int *type, double *range, double *delay, int *reference )
{
  char str[GPIB_STRMAX+1];

  if ( !gpib_write(&oszi,":timebase:sample?")) return 0;
  if ( !gpib_read(&oszi,str,GPIB_STRMAX)) return 0; 
  if (strstr(str,"REAL") != NULL) *type = HP_TYPE_REALTIME;
  else
    if (strstr(str,"REP") != NULL) *type = HP_TYPE_REPETITIVE;
  else
  {
    sprintf(errmsg,"\nIllegal timebase sample type (%s)\n",str);
    return 0;
  }

  if ( !gpib_write(&oszi,":timebase:range?")) return 0;
  if ( !gpib_read(&oszi,str,GPIB_STRMAX)) return 0;
  *range = atof(str);

  if ( !gpib_write(&oszi,":timebase:delay?")) return 0;
  if ( !gpib_read(&oszi,str,GPIB_STRMAX)) return 0;
  *delay = atof(str);

  if ( !gpib_write(&oszi,":timebase:reference?")) return 0;
  if ( !gpib_read(&oszi,str,GPIB_STRMAX)) return 0;

  if (strstr(str,"LEFT") != NULL) *reference = HP_REFERENCE_LEFT;
  else
    if (strstr(str,"CENT") != NULL) *reference = HP_REFERENCE_CENTER;
  else
    if (strstr(str,"RIGH") != NULL) *reference = HP_REFERENCE_RIGHT;
  else
  {
    sprintf(errmsg,"\nIllegal reference (%s)\n",str);
    return 0;
  }
  
  return 1;
}

static int HP_set_channel ( HP_Channel ch )
{ 
  if ( !gpib_write(&oszi,":channel%d:range %E",ch.no,ch.yrange))
  {
    sprintf(errmsg,"GPIB write error");
    return 0;
  }
  if ( !gpib_write(&oszi,":channel%d:offset %E",ch.no,ch.yoffset))
  {
    sprintf(errmsg,"GPIB write error");
    return 0;
  }
  if ( !gpib_write(&oszi,":channel%d:display %d",ch.no,((ch.active)?(1):(0))))
  {
    sprintf(errmsg,"GPIB write error");
    return 0;
  }

  return 1;
}

static int HP_get_channel ( HP_Channel *ch )
{
  char str[GPIB_STRMAX+1];
  
  if (!gpib_write(&oszi,":channel%d:range?",ch->no))
  {
    sprintf(errmsg,"GPIB write error");
    return 0;
  }
  if (!gpib_read(&oszi,str,GPIB_STRMAX))
  {
    sprintf(errmsg,"GPIB read error");
    return 0;
  }
  ch->yrange = atof(str);

  if (!gpib_write(&oszi,":channel%d:offset?",ch->no))
  {
    sprintf(errmsg,"GPIB write error");
    return 0;
  }
  if (!gpib_read(&oszi,str,GPIB_STRMAX))
  {
    sprintf(errmsg,"GPIB read error");
    return 0;
  }
  ch->yoffset = atof(str);

  if (!gpib_write(&oszi,":channel%d:display?",ch->no))
  {
    sprintf(errmsg,"GPIB write error");
    return 0;
  }
  if (!gpib_read(&oszi,str,GPIB_STRMAX))
  {
    sprintf(errmsg,"GPIB read error");
    return 0;
  }
  ch->active = atoi(str);

  return 1;
}

static int HP_set_acquire ( int type, int format, int points, int counts, int complete )
{
  char cmd[GPIB_STRMAX+1];
  
  /*  
  switch (type)
  {
    case HP_TYPE_NORMAL :
      sprintf(cmd,":timebase:sample realtime");
      if (!gpib_write(&oszi,cmd)) return 0;
      sprintf(cmd,":timebase:mode single");
      if (!gpib_write(&oszi,cmd)) return 0;
      sprintf(cmd,":acquire:type normal");
      if (!gpib_write(&oszi,cmd)) return 0;
    break;
    case HP_TYPE_AVERAGE :
      sprintf(cmd,":timebase:sample repetitive");
      if (!gpib_write(&oszi,cmd)) return 0;
      sprintf(cmd,":timebase:mode triggered");
      if (!gpib_write(&oszi,cmd)) return 0;
      sprintf(cmd,":acquire:type average");
      if (!gpib_write(&oszi,cmd)) return 0;
    break;
    case HP_TYPE_ENVELOPE :
      sprintf(errmsg,"\nacquire type 'envelope' not supported yet");
      return 0;
    break;
    case HP_TYPE_RAWDATA :
      sprintf(errmsg,"\nacquire type 'rawdata' not supported yet");
      return 0;
    break;
    default :
      sprintf(errmsg,"\nIllegal type (%d)\n",type);
      return 0;
    break;
  }
  */
 
  switch (format)
  {
    case HP_FORMAT_COMPRESSED :
      sprintf(cmd,":waveform:format comp");
      if (!gpib_write(&oszi,cmd)) return 0;
    break;
    case HP_FORMAT_BYTE :
      sprintf(cmd,":waveform:format byte");
      if (!gpib_write(&oszi,cmd)) return 0;
    break;
    case HP_FORMAT_WORD :
      sprintf(cmd,":waveform:format word");
      if (!gpib_write(&oszi,cmd)) return 0;
    break;
    case HP_FORMAT_ASCII :
      sprintf(cmd,":waveform:format ascii");
      if (!gpib_write(&oszi,cmd)) return 0;
    break;
    default :
      sprintf(errmsg,"\nIllegal format (%d)\n",format);
      return 0;
    break; 
  }

  /*
  sprintf(cmd,":acquire:points %d",((points<1024)?(500):(8000)));
  if (!gpib_write(&oszi,cmd)) return 0;
  sprintf(cmd,":acquire:count %d",counts);
  if (!gpib_write(&oszi,cmd)) return 0;
  sprintf(cmd,":acquire:complete %d",complete);
  if (!gpib_write(&oszi,cmd)) return 0;
  */

  return 1;
}

static int HP_get_acquire ( int *type, int *format, int *points, int *counts, int *complete )
{
  char  str[GPIB_STRMAX+1];
  
  if ( !gpib_write(&oszi,":acquire:type?")) return 0;
  if ( !gpib_read(&oszi,str,GPIB_STRMAX)) return 0;
  if (strstr(str,"NORM") != NULL) *type = HP_TYPE_NORMAL;
  else
    if (strstr(str,"AVER") != NULL) *type = HP_TYPE_AVERAGE;
  else
    if (strstr(str,"ENV") != NULL) *type = HP_TYPE_ENVELOPE;
  else
    if (strstr(str,"RAWD") != NULL) *type = HP_TYPE_RAWDATA;
  else
  {
    sprintf(errmsg,"\nIllegal type (%s)\n",str);
    return 0;
  }

  if ( !gpib_write(&oszi,":waveform:format?")) return 0;
  if ( !gpib_read(&oszi,str,GPIB_STRMAX)) return 0;
  if (strstr(str,"COMP") != NULL) *format = HP_FORMAT_COMPRESSED;
  else
    if (strstr(str,"BYTE") != NULL) *format = HP_FORMAT_BYTE;
  else
    if (strstr(str,"WORD") != NULL) *format = HP_FORMAT_WORD;
  else
    if (strstr(str,"ASC") != NULL) *format = HP_FORMAT_ASCII;
  else
  {
    sprintf(errmsg,"\nIllegal format (%s)\n",str);
    return 0;
  }

  if ( !gpib_write(&oszi,":acquire:points?")) return 0;
  if ( !gpib_read(&oszi,str,GPIB_STRMAX)) return 0;
  *points = atoi(str);

  if ( !gpib_write(&oszi,":acquire:count?")) return 0;
  if ( !gpib_read(&oszi,str,GPIB_STRMAX)) return 0;
  *counts = atoi(str);

  if ( !gpib_write(&oszi,":acquire:complete?")) return 0;
  if ( !gpib_read(&oszi,str,GPIB_STRMAX)) return 0;
  *complete = atoi(str);

  return 1;
}

static int HP_opc ( void )
{
  char opc_stat[GPIB_STRMAX+1];

  if ( !gpib_write(&oszi,"*OPC?")) return 0;
  if ( !gpib_read(&oszi,opc_stat,GPIB_STRMAX)) return 0;
  if (atoi(opc_stat) == 1) return 1;
  return 0;
  
}

static int HP_ter ( void )
{
  char ter_stat[GPIB_STRMAX+1];

  if ( !gpib_write(&oszi,":TER?")) return 0;
  if ( !gpib_read(&oszi,ter_stat,GPIB_STRMAX)) return 0;
  if (atoi(ter_stat) == 1) return 1;
  return 0;
  
}

static int HP_store_channel ( HP_Channel ch, Scheduler *parameters)
{
  char *waveform, *tp, token[6];
  char           datahead[11], eostr[2];
  int           *wave, wpmax, datamax, i;

  // if channel is disabled (->no data readout) quit immediately
  if (ch.enabled == 0)
    return 1;

  if (ch.active == 0) {  // channel is enabled by max but not(!) on the HP scope
    sprintf(errmsg,"HP_multi: ch%d enabled but not active",ch.no);
    return 0;
  }

  if ( !gpib_write(&oszi,":waveform:source channel%d",ch.no))  // select channel for read out
    return 0;
  
  if ( !gpib_write(&oszi,":waveform:data?"))  // ask for data
    return 0;

  if (aq_format < HP_FORMAT_ASCII)  // only BYTE, COMP & WORD start with header
  {
    for (i = 0; i < 11; datahead[i++] = '\0');
    if( !gpib_read (&oszi, datahead, 10))	// read data header info
    {
      sprintf(errmsg,"HP_multi[ch%d]: read error for 'datahead'",ch.no);
      return 0;
    }
    datamax = atoi(datahead+2);  // number of waveform data bytes 
  }
  else datamax = 6*aq_points;  // ASCII: 16bit data words ("0"..."32767") sep. by ','

  if ((waveform = (char *)calloc(datamax,sizeof(char))) == NULL)
  {
    sprintf(errmsg,"Couldn't allocate vector waveform (%d uchars) due to\n%s",datamax,strerror(errno));
    return 0;
  }

  switch (aq_format)
  {
    case HP_FORMAT_WORD :
      wpmax = datamax/2;
    break;
    case HP_FORMAT_BYTE :
    case HP_FORMAT_COMPRESSED :
      wpmax = datamax;
    break;
    case HP_FORMAT_ASCII :
    default :
      wpmax = aq_points;
    break;
  }

  if ((wave = (int *)calloc(wpmax,sizeof(int))) == NULL)
  {
    free(waveform);
    sprintf(errmsg,"Couldn't allocate vector wave (%d ints) due to\n%s",
            wpmax,strerror(errno));
    return 0;
  }

  printf("\nHP_multi[ch%d]: reading data (%d uchars for %d items)...",ch.no,datamax,wpmax);  
  if( !gpib_read(&oszi,waveform,datamax))
  {
    sprintf(errmsg,"HP_multi[ch%d]: read error for 'waveform'",ch.no);
    return 0;
  }
  
  // convert received GPIB-string 'waveform' into array of ints 'wave'
  switch (aq_format)
  {
    case HP_FORMAT_ASCII :
      i = 0; 
      while ((tp = strtok(((i==0)?(waveform):((unsigned char)NULL)),",")) != NULL)
      {
        strcpy(token,tp);
        wave[i++] = atoi(token);
      }
    break;
    case HP_FORMAT_WORD :
      for(i = 0; i < wpmax; i++)
        wave[i] = ((int)waveform[2*i] << 8) + (int)waveform[2*i+1];
    break;
    case HP_FORMAT_BYTE :
    case HP_FORMAT_COMPRESSED:
      for(i = 0; i < wpmax; wave[i] = (int)waveform[i++]);
    break;
  }

  eostr[0] = eostr[1] = '\0';  
  if( !gpib_read(&oszi,eostr,1))	// read data <terminator>
  {
    sprintf(errmsg,"HP_multi[ch%d]: read error for 'eostr'",ch.no);
    return 0;
  }

  parameters->store(wave,wpmax);	// save the wave data array

  free(waveform);
  free(wave);
  
  return 1;
}
/*************************************************/

//init function for HP-Scope
//called before meausurement starts
int HPmulti_init(List *device_data, Scheduler *parameters)
{
  // IEEE-init HP

  oszi = gpib_init("HP",IB_CLEAR|IB_REM|IB_IFC);
  if (oszi.stat != GPIB_OK)
  {
    sprintf(errmsg,"GPIB init error for '%s'@%02d",oszi.name,oszi.addr);
    if (!parameters->error(E_GPIB,errmsg))
      return 0;   //return FALSE -> measurement stops
  }
  else sprintf(errmsg,"\tNo error(s) occured\n");

  ch1.no = 1;
  ch2.no = 2;
  ch3.no = 3;
  ch4.no = 4;
  
  device_data->getvar(&TLAM,"all","timeout");

  device_data->getvar(&ch1.yrange,"HP_multi","ch1_yrange");  
  device_data->getvar(&ch1.yoffset,"HP_multi","ch1_yoffset");  
  device_data->getvar(&ch2.yrange,"HP_multi","ch2_yrange");  
  device_data->getvar(&ch2.yoffset,"HP_multi","ch2_yoffset");  
  device_data->getvar(&ch3.yrange,"HP_multi","ch3_yrange");  
  device_data->getvar(&ch3.yoffset,"HP_multi","ch3_yoffset");  
  device_data->getvar(&ch4.yrange,"HP_multi","ch4_yrange");  
  device_data->getvar(&ch4.yoffset,"HP_multi","ch4_yoffset");  

  device_data->getvar(&aq_type,"HP_multi","acq_type");  
  device_data->getvar(&aq_format,"HP_multi","acq_format");
  device_data->getvar(&aq_counts,"HP_multi","acq_count");  
  device_data->getvar(&aq_points,"HP_multi","acq_point");  
  device_data->getvar(&aq_complete,"HP_multi","acq_complete");  
  device_data->getvar(&aq_del,"HP_multi","acq_delay");  
  if (aq_del > 0) slowmode = 1;
  else slowmode = 0;
  
  device_data->getvar(&tb_type,"HP_multi","tb_type");  
  device_data->getvar(&tb_range,"HP_multi","tb_range");  
  device_data->getvar(&tb_delay,"HP_multi","tb_delay");  
  device_data->getvar(&tb_delref,"HP_multi","tb_delay_ref");  

  ch1.enabled = device_data->enabled("HP_multi","channel_1");
  ch2.enabled = device_data->enabled("HP_multi","channel_2");
  ch3.enabled = device_data->enabled("HP_multi","channel_3");
  ch4.enabled = device_data->enabled("HP_multi","channel_4");
  
  printf("\ninitializing HP...");
  
  if ( !gpib_write(&oszi,":run")) {
    if (!parameters->error(E_IB_WRITE,"GPIB write error"))
      return 0;   //return FALSE -> measurement stops
  }
  
  if ( !gpib_write(&oszi,":system:longform off;header off")) {
    if (!parameters->error(E_GPIB,"write error "))
      return 0;   //return FALSE -> measurement stops
  }
  
  if (HP_set_acquire(aq_type,aq_format,aq_points,aq_counts,aq_complete) != 1) {
    sprintf(qterrmsg,"\nHP_SET_ACQUIRE(): Error due to\n%s ",errmsg);
    if (!parameters->error(E_GPIB,qterrmsg))
      return 0;
  } 
  
  if (HP_get_timebase(&tb_type,&tb_range,&tb_delay,&tb_delref) != 1) {
    sprintf(qterrmsg,"\nHP_GET_TIMEBASE(): Error due to\n%s ",errmsg);
    if (!parameters->error(E_GPIB,qterrmsg))
      return 0;
  }  
  
  if (HP_get_channel(&ch1) != 1) {
    sprintf(qterrmsg,"\nHP_GET_CHANNEL(#1): Error due to\n%s ",errmsg);
    if (!parameters->error(E_GPIB,qterrmsg))
      return 0;
  }   
  if (HP_get_channel(&ch2) != 1) {
    sprintf(qterrmsg,"\nHP_GET_CHANNEL(#2): Error due to\n%s ",errmsg);
    if (!parameters->error(E_GPIB,qterrmsg))
      return 0;
  }   
  if (HP_get_channel(&ch3) != 1) {
    sprintf(qterrmsg,"\nHP_GET_CHANNEL(#3): Error due to\n%s ",errmsg);
    if (!parameters->error(E_GPIB,qterrmsg))
      return 0;
  }   
  if (HP_get_channel(&ch4) != 1) {
    sprintf(qterrmsg,"\nHP_GET_CHANNEL(#4): Error due to\n%s ",errmsg);
    if (!parameters->error(E_GPIB,qterrmsg))
      return 0;
  }   
  
  if (HP_get_acquire(&aq_type,&aq_format,&aq_points,&aq_counts,&aq_complete) != 1) {
    sprintf(qterrmsg,"\nHP_GET_ACQUIRE(): Error due to\n%s ",errmsg);
    if (!parameters->error(E_GPIB,qterrmsg))
      return 0;
  }
  
  printf("\b\b\b done");
  return 1;
}


int HPmulti_close(List *device_data, Scheduler *parameters)
{
  printf("\nclosing HP_multi ");
  
  if (HP_get_timebase(&tb_type,&tb_range,&tb_delay,&tb_delref) != 1) {
    sprintf(qterrmsg,"\nHP_GET_TIMEBASE(): Error due to\n%s ",errmsg);
    if (!parameters->error(E_GPIB,qterrmsg))
      return 0;
  }  
  
  if (HP_get_channel(&ch1) != 1) {
    sprintf(qterrmsg,"\nHP_GET_CHANNEL(#1): Error due to\n%s ",errmsg);
    if (!parameters->error(E_GPIB,qterrmsg))
      return 0;
  }   
  if (HP_get_channel(&ch2) != 1) {
    sprintf(qterrmsg,"\nHP_GET_CHANNEL(#2): Error due to\n%s ",errmsg);
    if (!parameters->error(E_GPIB,qterrmsg))
      return 0;
  }   
  if (HP_get_channel(&ch3) != 1) {
    sprintf(qterrmsg,"\nHP_GET_CHANNEL(#3): Error due to\n%s ",errmsg);
    if (!parameters->error(E_GPIB,qterrmsg))
      return 0;
  }   
  if (HP_get_channel(&ch4) != 1) {
    sprintf(qterrmsg,"\nHP_GET_CHANNEL(#4): Error due to\n%s ",errmsg);
    if (!parameters->error(E_GPIB,qterrmsg))
      return 0;
  }   
  
  if (HP_get_acquire(&aq_type,&aq_format,&aq_points,&aq_counts,&aq_complete) != 1) {
    sprintf(qterrmsg,"\nHP_GET_ACQUIRE(): Error due to\n%s ",errmsg);
    if (!parameters->error(E_GPIB,qterrmsg))
      return 0;
  }  
  
  // Keycode #39 - LOCAL
  if ( !gpib_write(&oszi,":system:key 39"))
    if (!parameters->error(E_IB_WRITE,"GPIB write error"))
      return 0;   //return FALSE -> measurement stops
  
  device_data->putvar(tb_type,"HP_multi","tb_type");  
  device_data->putvar(tb_range,"HP_multi","tb_range");  
  device_data->putvar(tb_delay,"HP_multi","tb_delay");  
  device_data->putvar(tb_delref,"HP_multi","tb_delay_ref");  

  device_data->putvar(ch1.active,"HP_multi","ch1_active");
  device_data->putvar(ch1.yrange,"HP_multi","ch1_yrange");
  device_data->putvar(ch1.yoffset,"HP_multi","ch1_yoffset");

  device_data->putvar(ch2.active,"HP_multi","ch2_active");
  device_data->putvar(ch2.yrange,"HP_multi","ch2_yrange");
  device_data->putvar(ch2.yoffset,"HP_multi","ch2_yoffset");

  device_data->putvar(ch3.active,"HP_multi","ch3_active");
  device_data->putvar(ch3.yrange,"HP_multi","ch3_yrange");
  device_data->putvar(ch3.yoffset,"HP_multi","ch3_yoffset");

  device_data->putvar(ch4.active,"HP_multi","ch4_active");
  device_data->putvar(ch4.yrange,"HP_multi","ch4_yrange");
  device_data->putvar(ch4.yoffset,"HP_multi","ch4_yoffset");

  device_data->putvar(aq_type,"HP_multi","acq_type");  
  device_data->putvar(aq_format,"HP_multi","acq_format");  
  device_data->putvar(aq_counts,"HP_multi","acq_count");  
  device_data->putvar(aq_points,"HP_multi","acq_point");  
  device_data->putvar(aq_complete,"HP_multi","acq_complete");  
  
  if ( !gpib_clear(&oszi)) {
    if (!parameters->error(E_GPIB,"Couldn't clear device HP_multi "))
      return 0;
  }
  
  return 1; //function has to return a vlue -> 1 otherwise measurement stops
}


//pre-trigger device function for HP
int HPmulti_pre(List *device_data, Scheduler *parameters) 
{  
  // Send 'digitize' / 'single shot' command to read the raw waveform data 

  switch (aq_type) {
  case HP_TYPE_SINGLE :
    printf("\nHP_multi: running single shot ");
    if ( !gpib_write(&oszi,":system:key 37"))   // keycode #37 - SINGLE
      if ( !parameters->error(E_IB_WRITE,"GPIB write error"))
	return 0;
    break;
  case HP_TYPE_TRIGGERED :
    printf("\nHP_multi: digitizing screen(s) ");
    if (ch1.enabled || ch2.enabled || ch3.enabled || ch4.enabled) {
      if ( !gpib_write(&oszi,":dig %s%s%s%s%s%s%s",
		       ch1.enabled?"chan1":"", \
		       (ch1.enabled&&(ch2.enabled||ch3.enabled||ch4.enabled))?",":"", \
		       ch2.enabled?"chan2":"", \
		       ((!ch1.enabled)&&ch2.enabled&&(ch3.enabled||ch4.enabled))?",":"", \
		       ch3.enabled?"chan3":"", \
		       ((!ch1.enabled)&&(!ch2.enabled)&&ch3.enabled&&ch4.enabled)?",":"", \
		       ch4.enabled?"chan4":""))
        if ( !parameters->error(E_IB_WRITE,"GPIB write error"))
	  return 0;
    }
    break;
  default :
    if ( !parameters->error(E_RANGE,"HP_multi: Illegal data acquisition type (%d)\n",aq_type))
      return 0;
    break;
  }
  
  printf("\nHP_multi: capturing data from active channel(s), please wait... ");
  
  return 1;
}


//device function for HP
int HPmulti(List *device_data, Scheduler *parameters) 
{
  int i;

  // wait until data is available
  // wait until digitizing / single shot completed
  if (aq_type == HP_TYPE_SINGLE) {
    i = 0;
    while ( !HP_ter() && ++i < TLAM) {
      putchar('.');
      if (slowmode) usleep((long unsigned int)(aq_del*aq_counts*1.0E+6));
    }
    
    if (i >= TLAM) {
      if (!parameters->error(E_TIMEOUT,"HP_multi timed out (%d)",i))
	return 0;
    }
  }
  else if (slowmode)
    usleep((long unsigned int)(aq_del*aq_counts*1.0E+6));

  printf("done ");
   
  if ( !HP_store_channel(ch1,parameters))
  {
    sprintf(qterrmsg,"HP_STORE_CHANNEL(#1): Error due to\n%s ",errmsg);
    if (!parameters->error(E_GPIB,qterrmsg))
      return 0;
  }
  //  for (i = 0; i < 500; i++) wave[i] = 255*i/499;
  //  parameters->store(wave,500);
  
  
  if ( !HP_store_channel(ch2,parameters))
  {
    sprintf(qterrmsg,"HP_STORE_CHANNEL(#2): Error due to\n%s ",errmsg);
    if (!parameters->error(E_GPIB,qterrmsg))
      return 0;
  }
  //  for (i = 0; i < 500; i++) wave[i] = 255*i/499;
  //  parameters->store(wave,500);

  
  if ( !HP_store_channel(ch3,parameters))
  {
    sprintf(qterrmsg,"HP_STORE_CHANNEL(#3): Error due to\n%s ",errmsg);
    if (!parameters->error(E_GPIB,qterrmsg))
      return 0;
  }
  //  for (i = 0; i < 500; i++) wave[i] = 255*i/499;
  //  parameters->store(wave,500);

  
  if ( !HP_store_channel(ch4,parameters))
  {
    sprintf(qterrmsg,"HP_STORE_CHANNEL(#4): Error due to\n%s ",errmsg);
    if (!parameters->error(E_GPIB,qterrmsg))
      return 0;
  }
  //  for (i = 0; i < 500; i++) wave[i] = 255*i/499;
  //  parameters->store(wave,500);

  // end up read out process: reset scope
  if ( !gpib_write(&oszi,"*cls"))  // clear memory
  {
    if (!parameters->error(E_IB_WRITE,"HP ibwrite err"))
      return 0;   //return FALSE -> measurement stops
  }
  printf("\nreadout HP done ");

  return 1; 
}








