/*
 * MAX : acs_stepper.cc, Wed Jan  2 14:44:59 CET 2002 -sg
 *
 * Device driver for ACS-Stepper (Auto Correlator Stage stepping motor)
 * OWIS LM 60 Step motor with ZFK Steppercontroller
 * only one stepper motor on one ZFK-module
 * based on the ZFK-Stepper driver for max2.0beta
 *
 * M. Dammasch   27.05.1999 portet to max2.1beta
 * 
 * This file is part of Max data acquisition software
 * (c) 1997-98 by C. Otto & M. Dammasch
 *
 * Max is free software; you can redistribute it and/or modify it
 * under the terms of the version 2 of GNU General Public License as
 * published by the Free Software Foundation.
 *
 * Max is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * (see the file COPYING) in this directory; if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

//this header has always to be included !!
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <unistd.h>
#include "list.hh"
#include "sched.hh"
#include "camacs.h"
#include "error.h"

//#define DEBUG_DEV_ACS 1

/*
    stepping frequencies [Hz]
    (different for each CAMAC-Modul - hardware set!)
*/
#define FREQ_START  1400  // static mode: const.: FREQ_START [Hz]
#define FREQ_MAX    2800  // dynamic mode: accel.: FREQ_START->FREQ_MAX [Hz]

#define WAIT  100  	// delay time[us] between CAMAC-commands
                        // (timing problem by the controller

/*
    Moving direction
*/
#define LEFT   (0x1 << 23)  // Bit W24 selected
#define RIGHT   0x0	    // Bit W24 not selected

/*
    Stepping mode
*/
#define WITH_DYNAMIC_CONTROLL 	 (0x1 << 22)  // Bit W23 selected
#define WITHOUT_DYNAMIC_CONTROLL  0x0         // Bit W23 not selected

/*
    Step motor driver bits, i.e address
*/
#define DRIVER_ADDR  0        // Bits W22 and W21 not selected

/*
	LAM-Mask Register bits
	Read: F(1)A(13), Write: F(17)A(13)
*/
#define LAM_AT_STEP_REG_EMPTY   0x1        // W1 selected
#define LAM_AT_LEFT_END_POS    (0x1 << 1)  // W2 selected
#define LAM_AT_RIGHT_END_POS   (0x1 << 2)  // W3 selected
#define LAM_AT_STOP_INPUT      (0x1 << 3)  // W4 selected

/*
    Global variable(s)
*/
static int TLAM;		// timeout (general value- device 'all')
static int N;			// Crate Slot No.
static int drv_addr;            // address of physical step motor driver
static int mode;                // stepping mode
static int direction;		// stepping direction
static int single_step;         // flag for single step mode
static int steps_inc      = 0;  // step size (per channel)
static int new_position   = 0;	// new position
static int position	  = 0;  // actual position
static int old_position   = 0;  // old position
static int start_position = 0;  // position at startup
static unsigned int lam_status     = 0;  // state of LAM-Request register
static unsigned int lam_mask       = 0;  // state of LAM-Mask register

int acs_stepper_init(List *device_data, Scheduler *parameters)
{
  unsigned int dummy;
  int i, qx=0;
	
  if (camac_init() != 1)
    return parameters->error(E_CAMAC,"Couldn't initialize device ");

  printf("\n\033[0minitializing acs_stepper ");	

  device_data->getvar(&TLAM,"all","timeout");  // get general timeout value
  //printf("\nTLAM = %d ",TLAM);

  device_data->getvar(&N,"acs_stepper","N");  // get number of CAMAC station
  //printf("\nN = %d ",N);

  /* set driver bits (fix hardware address for ACS-Stepper!) */
  drv_addr = DRIVER_ADDR;
  //printf("\nDriver = %d (0x%x) ",drv_addr,drv_addr);

  /* set stepping mode (fix for absolute position information!) */
  mode = WITHOUT_DYNAMIC_CONTROLL;
  single_step = 1;
  //printf("\nMode = %d (0x%x) [%s][%s] ",mode,mode,(mode==WITHOUT_DYNAMIC_CONTROLL)?("static"):("dynamic"),(single_step)?("single-step"):("multi-step"));

  device_data->getvar(&steps_inc,"acs_stepper","Increments");  // get moving steps per increment (per channel)
  //printf("\nIncrements = %d ",steps_inc);

  device_data->getvar(&start_position,"acs_stepper","Start_pos");  // get absolute origin
  //printf("\nStart_pos = %d ",start_position);

  device_data->getvar(&position,"acs_stepper","Actual_pos");  // get actual position relative to origin
  //printf("\nActual_pos = %d ",position);

  usleep(WAIT);  // timing problem by the controller
  dummy = LAM_AT_STOP_INPUT|LAM_AT_RIGHT_END_POS|LAM_AT_LEFT_END_POS|LAM_AT_STEP_REG_EMPTY;
#ifdef DEBUG_DEV_ACS
  printf("\nacs_stepper: F(23)A(12)[%d] ",dummy);
#endif
  if ((qx = camac(23,12,N,&dummy)) != 3) {  // clear LAM-Status register
    printf("\nacs_stepper: F(23)A(12)[%d] -> QX: %d ",dummy,qx);
    if (qx > 0) { // q or x not set - no response
      if (!parameters->error(E_DEVICE_NOT_FOUND,"acs_stepper: No stepper driver detected"))
	return 0;
    }
    else {  // qx==0
      if (!parameters->error(E_ALLFUCKEDUP,"acs_stepper: F(23)A(12)"))
	return 0;
    }	
  }	
#ifdef DEBUG_DEV_ACS			
  printf("-> QX: %d\tD: %d ",qx,dummy);
#endif

  usleep(WAIT);  // timing problem by the controller
  dummy = LAM_AT_STOP_INPUT|LAM_AT_RIGHT_END_POS|LAM_AT_LEFT_END_POS|LAM_AT_STEP_REG_EMPTY;
#ifdef DEBUG_DEV_ACS
  printf("\nacs_stepper: F(17)A(13)[%d] ",dummy);
#endif
  if ((qx = camac(17,13,N,&dummy)) != 3) {  // set LAM-Mask register
    printf("\nacs_stepper: F(17)A(13)[%d] -> QX: %d ",dummy,qx);
    if (!parameters->error(E_DEVICE_NOT_FOUND,"acs_stepper: No stepper driver detected"))
      return 0;
  }	
#ifdef DEBUG_DEV_ACS	
  printf("-> QX: %d\tD: %d => LAM_Mask: <%1d%1d%1d%1d> ",qx,dummy,dummy&LAM_AT_STOP_INPUT?1:0,dummy&LAM_AT_RIGHT_END_POS?1:0,dummy&LAM_AT_LEFT_END_POS?1:0,dummy&LAM_AT_STEP_REG_EMPTY?1:0);
#endif

  printf("\nacs_stepper: \033[32mChecking for LAM-Status - please wait...\033[39m ");

  printf("\nacs_stepper: Testing RIGHT direction ");
  direction = RIGHT;
  usleep(WAIT);  // timing problem by the controller
  dummy = direction+WITHOUT_DYNAMIC_CONTROLL+drv_addr+1;
#ifdef DEBUG_DEV_ACS
  printf("\nacs_stepper: F(16)A(0)[%d] ",dummy);
#endif
  if ((qx = camac(16,0,N,&dummy)) != 3) {  // set driver address
    printf("\nacs_stepper: F(16)A(0)[%d] -> QX: %d ",dummy,qx);
    if (!parameters->error(E_DEVICE_NOT_FOUND,"acs_stepper: No stepper driver detected"))
      return 0;
  }
#ifdef DEBUG_DEV_ACS
  printf("-> QX: %d\tD: %d ",qx,dummy);
#endif

  usleep(WAIT);  // timing problem by the controller
  i = 0;
  dummy = 0;
#ifdef DEBUG_DEV_ACS
  printf("\nacs_stepper: F(8)A(15)[%d] ",dummy);
#endif
  qx = camac(8,15,N,&dummy);  // test for LAM pending
  while ((++i < TLAM) && (!(qx & 2))) {
    usleep(WAIT);  // timing problem by the controller
    qx = camac(8,15,N,&dummy);
  }
#ifdef DEBUG_DEV_ACS
  printf("-[%d/%d]-> QX: %d\tD: %d ",i,TLAM,qx,dummy);
#endif

  if (i >= TLAM) {
    printf("\nacs_stepper: Stepper timed out - testing LEFT direction ");
    direction = LEFT;
    usleep(WAIT);  // timing problem by the controller
    dummy = direction+WITHOUT_DYNAMIC_CONTROLL+drv_addr+1;
#ifdef DEBUG_DEV_ACS
    printf("\nacs_stepper: F(16)A(0)[%d] ",dummy);
#endif
    if ((qx = camac(16,0,N,&dummy)) != 3) {  // set driver address
      printf("\nacs_stepper: F(16)A(0)[%d] -> QX: %d ",dummy,qx);
      if (!parameters->error(E_DEVICE_NOT_FOUND,"acs_stepper: No stepper driver detected"))
        return 0;
    }
#ifdef DEBUG_DEV_ACS
    printf("-> QX: %d\tD: %d ",qx,dummy);
#endif

    usleep(WAIT);  // timing problem by the controller
    i = 0;
    dummy = 0;
#ifdef DEBUG_DEV_ACS
    printf("\nacs_stepper: F(8)A(15)[%d] ",dummy);
#endif
    qx = camac(8,15,N,&dummy);  // test for LAM pending
    while ((++i < TLAM) && (!(qx & 2))) {
      usleep(WAIT);  // timing problem by the controller
      qx = camac(8,15,N,&dummy);
    }
#ifdef DEBUG_DEV_ACS
    printf("-[%d/%d]-> QX: %d\tD: %d ",i,TLAM,qx,dummy);
#endif

    if (i >= TLAM) {
    printf("\nacs_stepper: \033[35mSTOP\033[39m: Step motor timed out in both directions - giving up... ");
    if (!parameters->error(E_TIMEOUT,"acs_stepper: F(8)A(15)"))
      return 0;
    }
  }

  printf("\nacs_stepper: \033[32mLAM-Status available\033[39m ");

  usleep(WAIT);  // timing problem by the controller
  lam_status = 0;
#ifdef DEBUG_DEV_ACS
  printf("\nacs_stepper: F(1)A(14)[%d] ",lam_status);
#endif
  if ((qx = camac(1,14,N,&lam_status)) != 3) {  // read LAM-Request register
    printf("\nacs_stepper: F(1)A(14)[%d] -> QX: %d ",lam_status,qx);
    if (!parameters->error(E_DEVICE_NOT_FOUND,"acs_stepper: No stepper driver detected"))
      return 0;
  }
#ifdef DEBUG_DEV_ACS
  printf("-> QX: %d\tD: %d => LAM-Status: <%1d%1d%1d%1d> ",qx,lam_status,lam_status&LAM_AT_STOP_INPUT?1:0,lam_status&LAM_AT_RIGHT_END_POS?1:0,lam_status&LAM_AT_LEFT_END_POS?1:0,lam_status&LAM_AT_STEP_REG_EMPTY?1:0);
#endif

  /* check if stepper has finished -> refresh position info */
  if (lam_status & LAM_AT_STEP_REG_EMPTY) {
    position = (direction==RIGHT?position+1:position-1);
    device_data->putvar(position,"acs_stepper","Actual_pos");
  }

  usleep(WAIT);  // timing problem by the controller
  lam_mask = 0;
#ifdef DEBUG_DEV_ACS
  printf("\nacs_stepper: F(1)A(13)[%d] ",lam_mask);
#endif
  if ((qx = camac(1,13,N,&lam_mask)) != 3) {  // read LAM-Mask register
    printf("\nacs_stepper: F(1)A(13)[%d] -> QX: %d ",lam_mask,qx);
    if (!parameters->error(E_DEVICE_NOT_FOUND,"acs_stepper: No stepper driver detected"))
      return 0;
  }
#ifdef DEBUG_DEV_ACS
  printf("-> QX: %d\tD: %d => LAM-Mask: <%1d%1d%1d%1d> ",qx,lam_mask,lam_mask&LAM_AT_STOP_INPUT?1:0,lam_mask&LAM_AT_RIGHT_END_POS?1:0,lam_mask&LAM_AT_LEFT_END_POS?1:0,lam_mask&LAM_AT_STEP_REG_EMPTY?1:0);
  //printf("\nacs_stepper: LAM-Mask: LAM_AT_STOP_INPUT %s \nacs_stepper: LAM-Mask: LAM_AT_RIGHT_END_POS %s \nacs_stepper: LAM-Mask: LAM_AT_LEFT_END_POS %s \nacs_stepper: LAM-Mask: LAM_AT_STEP_REG_EMPTY %s ",lam_mask&LAM_AT_STOP_INPUT?"passed":"blocked",lam_mask&LAM_AT_RIGHT_END_POS?"passed":"blocked",lam_mask&LAM_AT_LEFT_END_POS?"passed":"blocked",lam_mask&LAM_AT_STEP_REG_EMPTY?"passed":"blocked");
#endif

  printf("\ninitializing acs_stepper completed ");

  return 1;
} // of init


int acs_stepper_close(List *device_data, Scheduler *parameters)
{
  printf("\nclosing acs_stepper -> Pos: %d ",position);

  device_data->putvar(position,"acs_stepper","Actual_pos");

  if (camac_close() != 1)
    return parameters->error(E_CAMAC,"Couldn't close device ");

  return 1;
}



//device function for acs_stepper
int acs_stepper_pre(List *device_data, Scheduler *parameters)
{
  int qx=0, i=0;
  int steps, si;
  unsigned int dummy;

  device_data->getvar(&old_position,"acs_stepper","Actual_pos");  // get actual position

  /* ************ calculate new position  ************* */

  new_position = (parameters->act_channel()-1) * steps_inc + start_position;

  steps = new_position - old_position;

  if (steps >= 0) {
    direction = RIGHT;
  }	
  else {	
    direction = LEFT;
  }	

#ifdef DEBUG_DEV_ACS
  printf("\n\033[7macs_stepper: Moving %d steps to %s ...\033[27m ",abs(steps),direction==RIGHT?"RIGHT":"LEFT");
#endif

  /* check if really something to do */

  if (steps == 0) {  // nothing to do -> quit...
    printf("\nacs_stepper: ready -> Pos: %d ",new_position);
    return 1;
  }

  /* get actual LAM-Status before doing anything */

  usleep(WAIT);  // timing problem by the controller
  lam_status = 0;
#ifdef DEBUG_DEV_ACS
  printf("\nacs_stepper: F(1)A(14)[%d] ",lam_status);
#endif
  if ((qx = camac(1,14,N,&lam_status)) != 3) {  // read LAM-Request register
    printf("\nacs_stepper: F(1)A(14)[%d] -> QX: %d ",lam_status,qx);
    if (!parameters->error(E_ALLFUCKEDUP,"acs_stepper: F(1)A(14)"))
      return 0;
  }
#ifdef DEBUG_DEV_ACS
  printf("-> QX: %d\tD: %d => LAM_Status: <%1d%1d%1d%1d> ",qx,lam_status,lam_status&LAM_AT_STOP_INPUT?1:0,lam_status&LAM_AT_RIGHT_END_POS?1:0,lam_status&LAM_AT_LEFT_END_POS?1:0,lam_status&LAM_AT_STEP_REG_EMPTY?1:0);
#endif

  /* check if moving is allowed */

  usleep(WAIT);  // timing problem by the controller
  if ((lam_status & LAM_AT_RIGHT_END_POS) && (direction == RIGHT)) {  // FATAL! -> exit?
    printf("\nacs_stepper: \033[35mSTOP\033[39m: Can't move further to right direction - giving up... ");
    if (!parameters->error(E_RANGE,"acs_stepper: STOP: Already at right end-position! "))
      return 0;
  }	
  if ((lam_status & LAM_AT_LEFT_END_POS) && (direction == LEFT)) {  // FATAL! -> exit?
    printf("\nacs_stepper: \033[35mSTOP\033[39m: Can't move further to left direction - giving up... ");
    if (!parameters->error(E_RANGE,"acs_stepper: STOP: Already at left end-position! "))
      return 0;
  }

  /* ************ moving to new position ************** */

  if (single_step) {  // single step mode
    position = old_position;

    for (si = 0; si < abs(steps); si++) {

      /* clear LAM-Status before moving */
      usleep(WAIT);  // timing problem by the controller
      dummy = LAM_AT_STOP_INPUT|LAM_AT_RIGHT_END_POS|LAM_AT_LEFT_END_POS|LAM_AT_STEP_REG_EMPTY;
#ifdef DEBUG_DEV_ACS
      printf("\nacs_stepper: F(23)A(12)[%d] ",dummy);
#endif
      if ((qx = camac(23,12,N,&dummy)) != 3) {  // clear LAM-Status register
	printf("\nacs_stepper: F(23)A(12)[%d] -> QX: %d ",dummy,qx);
	if (!parameters->error(E_ALLFUCKEDUP,"acs_stepper: F(23)A(12)"))
	  return 0;
      }
#ifdef DEBUG_DEV_ACS
      printf("-> QX: %d\tD: %d ",qx,dummy);
#endif

      /* move 1 single step */
      usleep(WAIT);  // timing problem by the controller
      dummy = direction + mode + drv_addr + 1;
#ifdef DEBUG_DEV_ACS
      printf("\n\033[7mStep-Reg: %d (0x%x) + %d (0x%x) + %d (0x%x) + 1 = %d (single step mode)\033[27m ",direction,direction,mode,mode,drv_addr,drv_addr,dummy);
      printf("\nacs_stepper: F(16)A(0)[%d] ",dummy);
#endif
      if ((qx = camac(16,0,N,&dummy)) != 3) {  // set step register
	printf("\nacs_stepper: F(16)A(0)[%d] -> QX: %d ",dummy,qx);
	if (! (qx&2)) { // q not set - no response
	  if (!parameters->error(E_DEVICE_NOT_FOUND,"acs_stepper: No stepper driver detected"))
	    return 0;
	}
	else {  // qx==0
	  if (!parameters->error(E_ALLFUCKEDUP,"acs_stepper: F(16)A(0)"))
	    return 0;
	}
      }
#ifdef DEBUG_DEV_ACS
      printf("-> QX: %d\tD: %d ",qx,dummy);
#endif

      /* now the stepper is moving... */		
      printf("\nacs_stepper: %s%d (%3s) -[%d/%d] ",(steps>0)?"+":"-",1,(direction==RIGHT)?"right":"left ",si,abs(steps));

      // wait until step motor has finished (approx.)
      usleep(WAIT);

      i = 0;
      dummy = 0;
#ifdef DEBUG_DEV_ACS
      printf("\nacs_stepper: F(8)A(15)[%d] ",dummy);
#endif
      qx = camac(8,15,N,&dummy);  // test for LAM pending
      while ((++i < TLAM) && (!(qx & 2))) {
	usleep(200);  // timing problem by the controller
	qx = camac(8,15,N,&dummy);
      }
#ifdef DEBUG_DEV_ACS
      printf("-[%d/%d]-> QX: %d\tD: %d ",i,TLAM,qx,dummy);
#endif
      if (i >= TLAM) {
	if (!parameters->error(E_TIMEOUT,"acs_stepper: F(8)A(15)"))
	  return 0;
      }

      usleep(WAIT);  // timing problem by the controller
      lam_status = 0;
#ifdef DEBUG_DEV_ACS
      printf("\nacs_stepper: F(1)A(14)[%d] ",lam_status);
#endif
      if ((qx = camac(1,14,N,&lam_status)) != 3) {  // read LAM-Request register
	printf("\nacs_stepper: F(1)A(14)[%d] -> QX: %d ",lam_status,qx);
	if (!parameters->error(E_ALLFUCKEDUP,"acs_stepper: F(1)A(14)"))
	  return 0;
      }
#ifdef DEBUG_DEV_ACS
      printf("-> QX: %d\tD: %d <%1d%1d%1d%1d> ",qx,lam_status,lam_status&LAM_AT_STOP_INPUT?1:0,lam_status&LAM_AT_RIGHT_END_POS?1:0,lam_status&LAM_AT_LEFT_END_POS?1:0,lam_status&LAM_AT_STEP_REG_EMPTY?1:0);
#endif

      /* check if stepper has finished -> refresh position info */
      if (lam_status & LAM_AT_STEP_REG_EMPTY) {
	position = (direction==RIGHT?position+1:position-1);
	printf("\b-> Pos: %d ",position);
      }

      device_data->putvar(position,"acs_stepper","Actual_pos");		

      /* check status before next move */
      if (lam_status & LAM_AT_STOP_INPUT) {
	printf("\nacs_stepper: \033[35mSTOP\033[39m: STOP-Input has been activated ");
	if (!parameters->error(E_RANGE,"acs_stepper: Stepper has stopped due to <STOP-Input activated> "))
	  return 0;	
      }
      if (lam_status & LAM_AT_RIGHT_END_POS) {
	if (direction == RIGHT) {  // FATAL! -> exit?
	  printf("\nacs_stepper: \033[35mSTOP\033[39m: Can't move further to right direction - giving up... ");
	  if (!parameters->error(E_RANGE,"acs_stepper: STOP: Already at right end-position! "))
	    return 0;
	}
	else {	
	  printf("\nacs_stepper: \033[35mWARNING\033[39m: Stepper at end-position RIGHT (but moving %s) ",direction==RIGHT?"RIGHT":"LEFT");
	}
      }
      if (lam_status & LAM_AT_LEFT_END_POS) {
	if (direction == LEFT) {  // FATAL! -> exit?
	  printf("\nacs_stepper: \033[35mSTOP\033[39m: Can't move further to left direction - giving up... ");
	  if (!parameters->error(E_RANGE,"acs_stepper: STOP: Already at left end-position! "))
	    return 0;
	}
	else {
	  printf("\nacs_stepper: \033[35mWARNING\033[39m: Stepper at end-position LEFT (but moving %s) ",direction==LEFT?"LEFT":"RIGHT");
	}
      }

    } /* end of for() */
  } /* end of if(single_step)*/	
  else {	
    printf("\nacs_stepper: \033[35mSTOP\033[39m: Illegal stepping mode (single step only)! ");
    parameters->error(E_ALLFUCKEDUP,"acs_stepper: Illegal stepping mode (single step only)!");
    return 0;	
  }	

  if (new_position != position)
    if (!parameters->error(E_RANGE,"acs_stepper: Stepper has finished before reaching final position (%d<>%d) ",position,new_position))
      return 0;

  return 1;	
} /* end of acs_stepper_pre */

int acs_stepper (List *device_data, Scheduler *parameters)
{
  // nothing to do after trigger occured...

  //device_data->putvar(position,"acs_stepper","Actual_pos");

  return 1;
}






