/*
 * MAX : sr400.cc, Fri May 24 11:15:00 CEST 2002
 *
 * Device driver for the SR400 Dual Channel Gated Photon Counter
 *
 * This file is part of Max data acquisition software
 * (c) 2002 S. Gerlach
 *
 * Max is free software; you can redistribute it and/or modify it
 * under the terms of the version 2 of GNU General Public License as
 * published by the Free Software Foundation.
 *
 * Max is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * (see the file COPYING) in this directory; if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>
#include <gpiberr.h>
#include <gpib.h>
#include "list.hh"
#include "sched.hh"
#include "error.hh"

#define DEBUG_DEV_SR400

// global variables
static GPIB_Device sr;
static double disc_level;	// -0.3 .. 0.3 (0.0002)		V
static int gate_mode;		// 0 - cw, 1 - FIXED
static int counting_mode;	// 0
static double counting_time;	// Gatetime 1e-4 .. 9e7		ms
static double dwell_time;	// 0.002 .. 60			s
static double gate_width;	// 5e-6 .. 999.2		ms
static double gate_delay;	// 0 .. 999.2			ms
static int period_number;	// 1 .. 2000
int count=0;

int sr400_init(List *device_data, Scheduler *parameters)
{	
  	printf("\ninitializing sr400\n");
  
	sr = gpib_init("sr400",IB_CLEAR|IB_REM|IB_IFC);

  	if (sr.stat != GPIB_OK) {
    		if ( !parameters->error(E_GPIB,"GPIB init err"))
      			return 0;
  	}

  	device_data->getvar(&disc_level,"sr400","disc_level");
  	device_data->getvar(&gate_mode,"sr400","gate_mode");
  	device_data->getvar(&counting_mode,"sr400","counting_mode");
  	device_data->getvar(&counting_time,"sr400","counting_time");
  	device_data->getvar(&dwell_time,"sr400","dwell_time");
  	device_data->getvar(&gate_width,"sr400","gate_width");
  	device_data->getvar(&gate_delay,"sr400","gate_delay");
  	device_data->getvar(&period_number,"sr400","period_number");

	// set counting_mode
	gpib_write(&sr,"CM0");
	// set gate_mode
	gpib_write(&sr,"GM0,%d",gate_mode);
	// set disc_level
	gpib_write(&sr,"DL0,%lf",disc_level);
	if (gate_mode == 0) {
		// set counter_preset
		gpib_write(&sr,"CP2,%lf",counting_time*10000.0);
		// set dwell_time
		gpib_write(&sr,"DT %lf",dwell_time);
	}
	else {
		gpib_write(&sr,"GW0,%lf",gate_width/1000.0);
		gpib_write(&sr,"GD0,%lf",gate_delay/1000.0);
		gpib_write(&sr,"CI2,3");
		gpib_write(&sr,"CP2,1e0");
		gpib_write(&sr,"NP%d",period_number);
	}
	
	// clear status
	gpib_write(&sr,"CR;CS");

	return 1;
}

int sr400_pre ( List *device_data, Scheduler *parameters )
{
	return 1;
}

int sr400 ( List *device_data, Scheduler *parameters )
{
	char counts[GPIB_STRMAX+1];

	if (gate_mode == 0) {
		//reset status
		gpib_write(&sr,"CR;CS");
		// sleep gate_time + 10 ms
		usleep(1000*(int)counting_time+10000);

		// get counts
		gpib_write(&sr,"QA1");
		printf("reading .. ");
		gpib_read(&sr,counts,GPIB_STRMAX);
	}
	else {
		// wait for triggered counts
		do {
			gpib_write(&sr,"QA");
			gpib_read(&sr,counts,GPIB_STRMAX);
			//sleep gate_time
			usleep(1000*(int)gate_width);
		} while (atoi(counts) <= 0 || atoi(counts) == count );
	}
	
	printf("%s",counts);
	count = atoi(counts);

	parameters->store(count);
	device_data->putvar(count,"sr400","count");

	return 1;
}

int sr400_close(List *device_data, Scheduler *parameters)
{
	printf("\nclosing sr400 ... done");
	gpib_write(&sr,"CR;CH;CL");
	return 1;
}
